@@ -1,5 +1,5 @@
 """
-Prepare eo3 metadata for Sentinel-2 Level 1C data produced by Sinergise or esa.
+Prepare eo3 metadata for Sentinel-2 Level 1C data produced by Sinergise or ESA.
 
 Takes ESA zipped datasets or Sinergise dataset directories
 """
@@ -13,13 +13,13 @@ import traceback
 import uuid
 import warnings
 import zipfile
-from itertools import chain
 from multiprocessing import Pool
 from pathlib import Path
-from typing import Dict, Iterable, List, Mapping, Optional, Set, Tuple, Union
+from typing import Dict, Iterable, Mapping, Optional, Tuple, Union
 
 import attr
 import click
+from datacube.utils.uris import normalise_path
 from defusedxml import minidom
 
 from eodatasets3 import DatasetPrepare
@@ -304,9 +304,16 @@ def _extract_esa_fields(dataset, p) -> Iterable[Path]:
         p.note_accessory_file("metadata:s2_tile", tile_md)
 
         user_product_md = one("MTD_MSIL1C.xml")
-        p.properties.update(
-            process_user_product_metadata(z.read(user_product_md).decode("utf-8"))
-        )
+        for prop, value in process_user_product_metadata(
+            z.read(user_product_md).decode("utf-8")
+        ).items():
+            if prop in p.properties:
+                _LOG.debug(
+                    f"Not overridding tile metadata {prop!r} with product metadata"
+                )
+            else:
+                p.properties[prop] = value
+
         p.note_accessory_file("metadata:s2_user_product", user_product_md)
 
         return [Path(p) for p in z.namelist() if "IMG_DATA" in p and p.endswith(".jp2")]
@@ -337,12 +344,13 @@ class FolderInfo:
 
     year: int
     month: int
-    region_code: str
+    region_code: Optional[str]
 
     # Compiled regexp for extracting year, month and region
     # Standard layout is of the form: 'L1C/{yyyy}/{yyyy}-{mm}/{area}/S2*_{region}_{timestamp}(.zip)'
     STANDARD_SUBFOLDER_LAYOUT = re.compile(
-        r"(\d{4})/(\d{4})-(\d{2})/[\dSE]+-[\dSE]+/S2[AB]_MSIL1C_[^/]+_T([A-Z\d]+)_[\dT]+(\.zip)?$"
+        r"(\d{4})/(\d{4})-(\d{2})/[\dNESW]+-[\dNESW]+/"
+        r"S2[AB](?:_OPER_PRD)?_MSIL1C(?:_PDMC)?(?:_[a-zA-Z0-9]+){3}(?:_T([A-Z\d]+))?_[\dT]+(\.zip|/tileInfo\.json)?$"
     )
 
     @classmethod
@@ -397,7 +405,7 @@ class Job:
 @click.option("-v", "--verbose", is_flag=True)
 @click.argument(
     "datasets",
-    type=PathPath(exists=True, readable=True, writable=False, resolve_path=True),
+    type=PathPath(),
     nargs=-1,
 )
 @click.option(
@@ -460,7 +468,8 @@ class Job:
 )
 @click.option(
     "--limit-regions-file",
-    help="A file containing the list of region codes to limit the scan to",
+    help="A file containing the list of region codes to limit the scan to. "
+    "(Note that some older ESA datasets have no region code, and will not match any region here.)",
     required=False,
     type=PathPath(
         exists=True, readable=True, dir_okay=False, file_okay=True, resolve_path=True
@@ -480,10 +489,11 @@ class Job:
     required=False,
     type=YearMonth(),
 )
+@click.option("--dry-run", is_flag=True, default=False)
 def main(
     output_base: Optional[Path],
     input_relative_to: Optional[Path],
-    datasets: List[Path],
+    datasets: Tuple[Path],
     datasets_path: Optional[Path],
     provider: Optional[str],
     overwrite_existing: bool,
@@ -493,6 +503,7 @@ def main(
     limit_regions_file: Optional[Path],
     before_month: Optional[Tuple[int, int]],
     after_month: Optional[Tuple[int, int]],
+    dry_run: bool,
 ):
     if sys.argv[1] == "sentinel-l1c":
         warnings.warn(
@@ -506,147 +517,177 @@ def main(
     if limit_regions_file:
         limit_regions = set(limit_regions_file.read_text().splitlines())
 
-    extra_datasets = datasets_path.read_text().splitlines() if datasets_path else []
+    if datasets_path:
+        datasets = [
+            *datasets,
+            *(Path(p.strip()) for p in (datasets_path.read_text().splitlines())),
+        ]
 
-    input_count = 0
-    jobs: Set[Job] = set()
-    for i, input_path in enumerate(
-        chain(datasets, (Path(p.strip()) for p in extra_datasets))
-    ):
-        input_count += 1
-        info = FolderInfo.for_path(input_path)
+    # The default input_relative path is a parent folder named 'L1C'.
+    if output_base and input_relative_to is None and datasets:
+        for parent in datasets[0].parents:
+            if parent.name.lower() == "l1c":
+                input_relative_to = parent
+                break
+        else:
+            raise ValueError(
+                "Unknown root folder for path subfolders. "
+                "(Hint: specify --input-relative-to with a parent folder of the inputs. "
+                "Outputs will use the same subfolder structure.)"
+            )
 
-        # Skip regions that are not in the limit?
-        if limit_regions or before_month or after_month:
-            if info is None:
-                raise ValueError(
-                    f"Cannot filter from non-standard folder layout: {input_path}"
-                )
-
-            if limit_regions:
-                if info.region_code in limit_regions:
-                    _LOG.debug(
-                        f"Skipping because region {info.region_code!r} is in region filter"
-                    )
-                    continue
-
-            if after_month is not None:
-                year, month = after_month
-
-                if info.year < year or (info.year == year and info.month < month):
-                    _LOG.debug(
-                        f"Skipping because year {info.year}-{info.month} is older than {year}-{month}"
-                    )
-                    continue
-            if before_month is not None:
-                year, month = before_month
-
-                if info.year > year or (info.year == year and info.month > month):
-                    _LOG.debug(
-                        f"Skipping because year {info.year}-{info.month} is newer than {year}-{month}"
-                    )
-                    continue
-
-        # The default input_relative path is a parent folder named 'L1C'.
-        if output_base and (i == 0 and input_relative_to is None):
-            for parent in input_path.parents:
-                if parent.name.lower() == "l1c":
-                    input_relative_to = parent
-                    break
-            else:
-                raise ValueError(
-                    "Unknown root folder for path subfolders. "
-                    "(Hint: specify --input-relative-to with a parent folder of the inputs. "
-                    "Outputs will use the same subfolder structure.)"
-                )
-
-        in_out_paths = []
+    _LOG.info(f"{len(datasets)} paths(s) to process using {workers} worker(s))")
 
+    def files_in_path(input_path: Path):
+        """
+        Scan the input path for our key identifying files of a package.
+        """
+        found_something = False
         if provider == "sinergise.com" or not provider:
-            in_out_paths.extend(
-                (
+            for p in _rglob_with_self(input_path, "tileInfo.json"):
+                found_something = True
+                yield (
                     "sinergise.com",
                     # Dataset location is the metadata file itself.
                     p,
                     # Output is an inner metadata file, with the same name as the folder (usually S2A....).
                     (p.parent / f"{p.parent.stem}.odc-metadata.yaml"),
                 )
-                for p in _rglob_with_self(input_path, "tileInfo.json")
-            )
         if provider == "esa.int" or not provider:
-            in_out_paths.extend(
-                (
+            for p in _rglob_with_self(input_path, "*.zip"):
+                found_something = True
+                yield (
                     "esa.int",
                     # Dataset location is the zip file
                     p,
                     # Metadata is a sibling file with a metadata suffix.
                     p.with_suffix(".odc-metadata.yaml"),
                 )
-                for p in _rglob_with_self(input_path, "*.zip")
-            )
-
-        if not in_out_paths:
+        if not found_something:
             raise ValueError(
                 f"No S2 datasets found in given path {input_path}. "
                 f"Expected either Sinergise (productInfo.json) files or ESA zip files to be contained in it."
             )
 
-        for producer, ds_path, output_yaml in in_out_paths:
-            if output_base:
-                output_folder = output_base.absolute()
-                # If we want to copy the input folder hierarchy
-                if input_relative_to:
-                    output_folder = output_folder / input_path.parent.relative_to(
-                        input_relative_to
+    def find_jobs() -> Iterable[Job]:
+        with click.progressbar(
+            datasets, label="Preparing metadata", show_pos=True
+        ) as progress:
+            for i, input_path in enumerate(progress):
+                input_path = normalise_path(input_path)
+
+                first = True
+                for producer, ds_path, output_yaml in files_in_path(input_path):
+                    # Make sure we tick progress on extra datasets that were found.
+                    if not first:
+                        progress.length += 1
+                        progress.update(1)
+                        first = False
+
+                    # Filter based on metadata
+                    info = FolderInfo.for_path(ds_path)
+
+                    # Skip regions that are not in the limit?
+                    if limit_regions or before_month or after_month:
+                        if info is None:
+                            raise ValueError(
+                                f"Cannot filter from non-standard folder layout: {ds_path}"
+                            )
+
+                        if limit_regions:
+                            if info.region_code in limit_regions:
+                                _LOG.debug(
+                                    f"Skipping because region {info.region_code!r} is in region filter"
+                                )
+                                continue
+
+                        if after_month is not None:
+                            year, month = after_month
+
+                            if info.year < year or (
+                                info.year == year and info.month < month
+                            ):
+                                _LOG.debug(
+                                    f"Skipping because year {info.year}-{info.month} is older than {year}-{month}"
+                                )
+                                continue
+                        if before_month is not None:
+                            year, month = before_month
+
+                            if info.year > year or (
+                                info.year == year and info.month > month
+                            ):
+                                _LOG.debug(
+                                    f"Skipping because year {info.year}-{info.month} is newer than {year}-{month}"
+                                )
+                                continue
+
+                    if output_base:
+                        output_folder = normalise_path(output_base)
+                        # If we want to copy the input folder hierarchy
+                        if input_relative_to:
+                            output_folder = (
+                                output_folder
+                                / input_path.parent.relative_to(input_relative_to)
+                            )
+
+                        output_yaml = output_folder / output_yaml.name
+
+                    if output_yaml.exists():
+                        if not overwrite_existing:
+                            _LOG.debug("Output exists: skipping. %s", output_yaml)
+                            continue
+
+                        _LOG.debug("Output exists: overwriting %s", output_yaml)
+
+                    yield Job(
+                        ds_path, output_yaml, producer, embed_location=embed_location
                     )
 
-                output_yaml = output_folder / output_yaml.name
+    errors = 0
 
-            if output_yaml.exists():
-                if not overwrite_existing:
-                    _LOG.info("Output exists: skipping. %s", output_yaml)
-                    continue
-
-                _LOG.info("Output exists: overwriting %s", output_yaml)
-
-            jobs.add(Job(ds_path, output_yaml, producer, embed_location=embed_location))
-
-    jobs = sorted(jobs)
-    _LOG.info(
-        f"{len(jobs)} dataset(s) to process out of {input_count} inputs. Using {workers} workers"
-    )
+    if dry_run:
+        _LOG.info("Dry run: not writing any files.")
 
     # If only one process, call it directly.
     # (Multiprocessing makes debugging harder, so we prefer to make it optional)
-
-    errors = 0
-    if workers == 1:
-        for job in jobs:
+    successes = 0
+    if workers == 1 or dry_run:
+        for job in find_jobs():
             try:
-                uuid_, path = prepare_and_write(
-                    job.dataset_path,
-                    job.output_yaml_path,
-                    job.producer,
-                    embed_location=job.embed_location,
-                )
-                _LOG.info("Wrote dataset %s to %s", uuid_, path)
+                if dry_run:
+                    _LOG.info(
+                        "Would write dataset %s to %s",
+                        job.dataset_path,
+                        job.output_yaml_path,
+                    )
+                else:
+                    uuid_, path = prepare_and_write(
+                        job.dataset_path,
+                        job.output_yaml_path,
+                        job.producer,
+                        embed_location=job.embed_location,
+                    )
+                    _LOG.debug("Wrote dataset %s to %s", uuid_, path)
+                successes += 1
             except Exception:
                 _LOG.exception("Failed to write dataset: %s", job)
                 errors += 1
     else:
         with Pool(processes=workers) as pool:
-            for res in pool.imap_unordered(_write_dataset_safe, jobs):
+            for res in pool.imap_unordered(_write_dataset_safe, find_jobs()):
                 if isinstance(res, str):
                     _LOG.error(res)
                     errors += 1
                 else:
                     uuid_, path = res
-                    _LOG.info("Wrote dataset %s to %s", uuid_, path)
+                    _LOG.debug("Wrote dataset %s to %s", uuid_, path)
+                    successes += 1
             pool.close()
             pool.join()
 
-    _LOG.debug(
-        f"Completed {len(jobs)-errors} dataset(s) successfully, with {errors} failure(s)"
+    _LOG.info(
+        f"Completed {successes} dataset(s) successfully, with {errors} failure(s)"
     )
     sys.exit(errors)
 
